package nu.zoom.dsl.jte;

import freemarker.template.Configuration;
import freemarker.template.Template;
import freemarker.template.TemplateException;
import freemarker.template.TemplateExceptionHandler;
import nu.zoom.dsl.ast.DocumentNode;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Stream;

public class Generator {
    private final Path templatesDir ;
    private final DocumentNode data;
    private final Path outputDir ;
    private final Configuration cfg;

    public Generator(Path templatesDir, DocumentNode data, Path outputDir) throws IOException {
        this.templatesDir = Objects.requireNonNull(templatesDir);
        this.data = Objects.requireNonNull(data);
        this.outputDir = Objects.requireNonNull(outputDir);
        this.cfg = new Configuration(Configuration.VERSION_2_3_34);
        cfg.setDirectoryForTemplateLoading(templatesDir.toFile());
        cfg.setDefaultEncoding("UTF-8");
        cfg.setTemplateExceptionHandler(TemplateExceptionHandler.RETHROW_HANDLER);
        cfg.setLogTemplateExceptions(false);
        cfg.setWrapUncheckedExceptions(true);
        cfg.setFallbackOnNullLoopVariable(false);
    }

    public List<Path> generate() throws IOException, TemplateException {
        try (Stream<Path> files = Files.list(templatesDir)) {
            List<Path> templates = files.filter(p -> p.toString().endsWith(".ftl")).toList() ;
            ArrayList<Path> out = new ArrayList<>();
            for (Path template : templates) {
                // TODO file ending
                Path outpath = outputDir.resolve(outputFilenameFromTemplate(template.getFileName()));
                Template ftl = this.cfg.getTemplate(template.getFileName().toString()) ;
                try (var outw = Files.newBufferedWriter(outpath, StandardCharsets.UTF_8)) {
                    ftl.process(this.data, outw);
                    out.add(outpath);
                }
            }
            return out ;
        }
    }

    private String outputFilenameFromTemplate(Path template) {
        return template.getFileName().toString().replace(".ftl", ".scala");
    }
}
